"use strict";
exports.__esModule = true;
var diff = require("fast-diff");
var report_delete_1 = require("./report-delete");
var report_insert_1 = require("./report-insert");
var report_replace_1 = require("./report-replace");
// modified from https://github.com/prettier/eslint-plugin-prettier/blob/6c351c3/eslint-plugin-prettier.js#L111-L198
var line_ending_regex = /\r\n|[\r\n\u2028\u2029]/;
function report_differences(context, formatted) {
    var source = context.sourceFile.getFullText();
    var results = diff(source, formatted);
    var batch = [];
    var offset = 0; // NOTE: INSERT never advances the offset.
    while (results.length > 0) {
        var result = results.shift();
        var operation = result[0], text = result[1];
        switch (operation) {
            case diff.INSERT:
            case diff.DELETE:
                batch.push(result);
                break;
            case diff.EQUAL:
                // tslint:disable-next-line:early-exit
                if (results.length > 0) {
                    if (batch.length > 0) {
                        if (line_ending_regex.test(text)) {
                            flush();
                            offset += text.length;
                        }
                        else {
                            batch.push(result);
                        }
                    }
                    else {
                        offset += text.length;
                    }
                }
                break;
            default:
                throw new Error("Unexpected fast-diff operation '" + operation + "'");
        }
        if (batch.length > 0 && results.length === 0) {
            flush();
        }
    }
    function flush() {
        var ahead_delete_text = '';
        var ahead_insert_text = '';
        while (batch.length > 0) {
            var next = batch.shift();
            var operation = next[0], text = next[1];
            switch (operation) {
                case diff.INSERT:
                    ahead_insert_text += text;
                    break;
                case diff.DELETE:
                    ahead_delete_text += text;
                    break;
                case diff.EQUAL:
                    ahead_delete_text += text;
                    ahead_insert_text += text;
                    break;
                default:
                    throw new Error("Unexpected fast-diff operation '" + operation + "'");
            }
        }
        if (ahead_delete_text.length > 0 && ahead_insert_text.length > 0) {
            report_replace_1.report_replace(context, offset, ahead_delete_text, ahead_insert_text);
        }
        else if (ahead_delete_text.length === 0 && ahead_insert_text.length > 0) {
            report_insert_1.report_insert(context, offset, ahead_insert_text);
        }
        else if (ahead_delete_text.length > 0 && ahead_insert_text.length === 0) {
            report_delete_1.report_delete(context, offset, ahead_delete_text);
        }
        offset += ahead_delete_text.length;
    }
}
exports.report_differences = report_differences;
